Index: manager.c
===================================================================
RCS file: /usr/cvsroot/asterisk/manager.c,v
retrieving revision 1.38
diff -u -r1.38 manager.c
--- manager.c	7 Feb 2004 07:13:33 -0000	1.38
+++ manager.c	25 Feb 2004 01:12:13 -0000
@@ -62,6 +62,36 @@
 static struct manager_action *first_action = NULL;
 static ast_mutex_t actionlock = AST_MUTEX_INITIALIZER;
 
+
+
+
+static int ast_carefulwrite(int fd, char *s, int len, int timeoutms) 
+{
+	/* Try to write string, but wait no more than ms milliseconds
+	   before timing out */
+	int res;
+	struct timeval tv;
+	fd_set fds;
+	while(len) {
+		res = write(fd, s, len);
+		if ((res < 0) && (errno != EAGAIN)) {
+			return -1;
+		}
+		if (res < 0) res = 0;
+		len -= res;
+		s += res;
+		tv.tv_sec = timeoutms / 1000;
+		tv.tv_usec = timeoutms % 1000;
+		FD_ZERO(&fds);
+		FD_SET(fd, &fds);
+		/* Wait until writable again */
+		res = select(fd + 1, NULL, &fds, NULL, &tv);
+		if (res < 1)
+			return -1;
+	}
+}
+
+
 static int handle_showmancmds(int fd, int argc, char *argv[])
 {
 	struct manager_action *cur = first_action;
@@ -207,7 +237,9 @@
 	char *pass = astman_get_header(m, "Secret");
 	char *authtype = astman_get_header(m, "AuthType");
 	char *key = astman_get_header(m, "Key");
-
+	char *events = astman_get_header(m, "Events");
+	int events_on = events ? ast_true(events) : 1;
+	
 	cfg = ast_load("manager.conf");
 	if (!cfg)
 		return -1;
@@ -272,6 +304,7 @@
 		s->readperm = get_perm(ast_variable_retrieve(cfg, cat, "read"));
 		s->writeperm = get_perm(ast_variable_retrieve(cfg, cat, "write"));
 		ast_destroy(cfg);
+		s->events_on=events_on;
 		return 0;
 	}
 	ast_log(LOG_NOTICE, "%s tried to authenticate with non-existant user '%s'\n", inet_ntoa(s->sin.sin_addr), user);
@@ -285,6 +318,23 @@
 	return 0;
 }
 
+
+static int action_events_on(struct mansession *s, struct message *m)
+{
+	s->events_on=1;
+	astman_send_response(s, m, "Events On", NULL);
+	return 0;
+}
+
+
+static int action_events_off(struct mansession *s, struct message *m)
+{
+	s->events_on=0;
+	astman_send_response(s, m, "Events Off", NULL);
+	return 0;
+}
+
+
 static int action_logoff(struct mansession *s, struct message *m)
 {
 	astman_send_response(s, m, "Goodbye", "Thanks for all the fish.");
@@ -758,6 +808,7 @@
 		fcntl(as, F_SETFL, flags | O_NONBLOCK);
 		ast_mutex_init(&s->lock);
 		s->fd = as;
+		s->events_on = 1;
 		ast_mutex_lock(&sessionlock);
 		s->next = sessions;
 		sessions = s;
@@ -778,14 +829,16 @@
 	ast_mutex_lock(&sessionlock);
 	s = sessions;
 	while(s) {
-		if ((s->readperm & category) == category) {
+		if (((s->readperm & category) == category) && s->events_on) {
 			ast_mutex_lock(&s->lock);
 			if (!s->blocking) {
 				ast_cli(s->fd, "Event: %s\r\n", event);
 				va_start(ap, fmt);
 				vsnprintf(tmp, sizeof(tmp), fmt, ap);
 				va_end(ap);
-				write(s->fd, tmp, strlen(tmp));
+
+				ast_carefulwrite(s->fd,tmp,strlen(tmp),100);
+				/*write(s->fd, tmp, strlen(tmp));*/
 				ast_cli(s->fd, "\r\n");
 			}
 			ast_mutex_unlock(&s->lock);
@@ -871,6 +924,8 @@
 	if (!registered) {
 		/* Register default actions */
 		ast_manager_register( "Ping", 0, action_ping, "Ping" );
+		ast_manager_register( "EventsOn", 0, action_events_on, "EventsOn" );
+		ast_manager_register( "EventsOff", 0, action_events_off, "EventsOff" );
 		ast_manager_register( "Logoff", 0, action_logoff, "Logoff Manager" );
 		ast_manager_register( "Hangup", EVENT_FLAG_CALL, action_hangup, "Hangup Channel" );
 		ast_manager_register( "Status", EVENT_FLAG_CALL, action_status, "Status" );
Index: include/asterisk/manager.h
===================================================================
RCS file: /usr/cvsroot/asterisk/include/asterisk/manager.h,v
retrieving revision 1.7
diff -u -r1.7 manager.h
--- include/asterisk/manager.h	8 Sep 2003 16:44:36 -0000	1.7
+++ include/asterisk/manager.h	25 Feb 2004 01:12:13 -0000
@@ -66,7 +66,7 @@
 	int writeperm;
 	char inbuf[MAX_LEN];
 	int inlen;
-	
+	int events_on;
 	struct mansession *next;
 };
 
